#chtNounConstruct.py
#A plugin allowing toggling on and off the Chinese character definition and noun construction functionality.
#Written by Michael Chen <m11chen@gmail.com>

import globalPluginHandler
import ui
import config
from config import conf
import speech
import speechDictHandler

confSect = conf.get("chtNounConstruct")

if not confSect:
	confSect = conf["chtNounConstruct"] = {}
	conf["chtNounConstruct"]["Enabled"] = "Off"

def _customSpeakSpellingGen(text):
	textLength=len(text)
	synth=speech.getSynth()
	synthConfig=config.conf["speech"][synth.name]
	for count,char in enumerate(text): 
		uppercase=char.isupper()
		charIsAlNum=char.isalnum()
		char=speech.processSymbol(char)
		if uppercase and synthConfig["sayCapForCapitals"]:
			sChar = char
			char=_("cap %s")%char
		if uppercase and synth.isSupported("pitch") and synthConfig["raisePitchForCapitals"]:
			oldPitch=synthConfig["pitch"]
			synth.pitch=max(0,min(oldPitch+synthConfig["capPitchChange"],100))
		index=count+1
		speech.log.io("Speaking character %r"%char)
		if conf["chtNounConstruct"]["Enabled"] == "On" and charIsAlNum:
			if uppercase and synthConfig["sayCapForCapitals"]:
				synth.speakText("Cap "+speechDictHandler.processText(sChar+"_____"),index=index)
			else:
				synth.speakText(char+" "+speechDictHandler.processText(char+"_____"),index=index)
		elif len(char) == 1 and synthConfig["useSpellingFunctionality"]:
			synth.speakCharacter(char,index=index)
		else:
			synth.speakText(char,index=index)
		if uppercase and synth.isSupported("pitch") and synthConfig["raisePitchForCapitals"]:
			synth.pitch=oldPitch
		while textLength>1 and (speech.isPaused or speech.getLastSpeechIndex()!=index):
			yield
			yield
		if uppercase and  synthConfig["beepForCapitals"]:
			speech.tones.beep(2000,50)

class GlobalPlugin(globalPluginHandler.GlobalPlugin):

	def __init__(self):
		super(GlobalPlugin, self).__init__()
		self._oldSpeakSpellingGen = speech._speakSpellingGen
		speech._speakSpellingGen = _customSpeakSpellingGen

	def script_ToggleNounConstruction(self,gesture):
		if conf["chtNounConstruct"]["Enabled"] == "Off":
			onOff=_("On")
			conf["chtNounConstruct"]["Enabled"] = "On"
		else:
			onOff=_("Off")
			conf["chtNounConstruct"]["Enabled"] = "Off"
		ui.message(_("noun construction")+" "+onOff)
	script_ToggleNounConstruction.__doc__=_("Toggles on or off speaking of noun constructions from current character")

	def terminate(self):
		speech._speakSpellingGen = self._oldSpeakSpellingGen
		super(GlobalPlugin, self).terminate()

#: Dictionary for key bindings
	__gestures = {
		"kb:NVDA+0": "ToggleNounConstruction",
	}
